"""Chat with local and remote LLMs in the context of the wordslab-notebooks environment"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/02_chat.ipynb.

# %% auto 0
__all__ = ['NbChat']

# %% ../nbs/02_chat.ipynb 3
import os
from ollama import chat
import openai

from .core import WordslabNotebooks
from .context import get_notebook_context

from IPython.display import display, Markdown, clear_output

# %% ../nbs/02_chat.ipynb 49
class NbChat:
    def __init__(self, model=None):
        if model is None:
            wlnb = WordslabNotebooks()
            self.model = wlnb.default_model_chat
        else:
            self.model = model

        self.prompt_template = """You are an AI assistant designed to help the user learn and solve problems interactively.
You run in an interactive Jupyter notebook environment where the user can write code, take notes, and chat with you.
You work with the user step-by-step rather than just giving complete answers. You are especially good at:
- Breaking down complex topics into manageable pieces
- Helping the user work through coding problems in Python
- Encouraging the user to try things himself, with guidance when he needs it
- Adapting to the user level and interests
You are designed to be collaborative - you ask questions, check the user understanding, and let him explore ideas rather than just lecturing. 
You can help with teaching, coding, problem-solving, research, and creative projects.
What sets you apart is your teaching approach - you focus on helping the user develop his skills rather than just giving him answers. 
You provide information in small chunks, check in frequently to see if things make sense, and encourage the user to try things himself.

# Jupyter notebook - all cells above the user instruction in XML

{notebook_context}

# User Instruction - in the last code cell of the notebook

Execute this user instruction in the context of the code cells above:

{user_instruction}
"""

    async def __call__(self, user_instruction, timeout=1):
        notebook_context = await get_notebook_context(timeout=timeout)
        prompt = self.prompt_template.format(user_instruction=user_instruction, notebook_context=notebook_context)
        stream = chat(model=self.model, messages=[{'role': 'user', 'content': prompt}], stream=True)
        streamed_text = ""
        for chunk in stream:
            streamed_text += chunk['message']['content']    
            clear_output(wait=True)
            display(Markdown(streamed_text))



"""Access wordslab-notebooks version, platform, urls, ports, directories (install / apps / data / models), and pre-installed models."""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_core.ipynb.

# %% auto 0
__all__ = ['WordslabNotebooks']

# %% ../nbs/00_core.ipynb 3
import os

# %% ../nbs/00_core.ipynb 5
class WordslabNotebooks:
    def __init__(self):
        # wordslab-notebooks version and platform
        self.version = os.environ["WORDSLAB_VERSION"]
        self.platform = os.environ["WORDSLAB_PLATFORM"]

        # wordslab-notebooks external urls
        self.url_dashboard = os.environ["DASHBOARD_URL"]
        self.url_openwebui = os.environ["OPENWEBUI_URL"]
        self.url_jupyterlab = os.environ["JUPYTERLAB_URL"]
        self.url_vscode = os.environ["VSCODE_URL"]
        self.url_userapp1 = os.environ["USER_APP1_URL"]
        self.url_userapp2 = os.environ["USER_APP2_URL"]
        self.url_userapp3 = os.environ["USER_APP3_URL"]
        self.url_userapp4 = os.environ["USER_APP4_URL"]
        self.url_userapp5 = os.environ["USER_APP5_URL"]

        # wordslab-notebooks internal ports
        self.port_ollama = 11434
        self.port_vllm = 8000
        self.port_docling = 5001   
        self.port_userapp1 = os.environ["USER_APP1_PORT"]
        self.port_userapp2 = os.environ["USER_APP2_PORT"]
        self.port_userapp3 = os.environ["USER_APP3_PORT"]
        self.port_userapp4 = os.environ["USER_APP4_PORT"]
        self.port_userapp5 = os.environ["USER_APP5_PORT"]
        
        # wordslab-notebooks install directories
        self.dir_home = os.environ["WORDSLAB_HOME"]
        self.dir_scripts = os.environ["WORDSLAB_SCRIPTS"]     
        self.dir_python = os.environ["UV_PYTHON_INSTALL_DIR"]  
        self.dir_workspace = os.environ["WORDSLAB_WORKSPACE"]
        self.dir_models = os.environ["WORDSLAB_MODELS"]

        # wordslab-notebooks applications install directories
        self.dir_openwebui = os.environ["OPENWEBUI_ENV"]
        self.dir_jupyterlab = os.environ["JUPYTERLAB_ENV"]
        self.dir_vscode = os.environ["VSCODE_DIR"]
        self.dir_ollama = os.environ["OLLAMA_DIR"]
        self.dir_docling = os.environ["DOCLING_ENV"]  

        # wordslab-notebooks applications data directories
        self.dir_openwebui_data = os.environ["OPENWEBUI_DATA"]
        self.dir_jupyterlab_data = os.environ["JUPYTER_DATA"]
        self.dir_vscode_data = os.environ["VSCODE_DATA"]
        self.dir_docling_data = os.environ["DOCLING_DATA"]
        
        # wordslab-notebooks models directories
        self.dir_models_ollama = os.environ["OLLAMA_MODELS"] 
        self.dir_models_vllm = os.environ["HF_HOME"]
        self.dir_models_hugginface = os.environ["HF_HOME"]
        self.dir_models_fastai = os.environ["FASTAI_HOME"] 
        self.dir_models_pytorch = os.environ["TORCH_HOME"] 
        self.dir_models_keras = os.environ["KERAS_HOME"]
        self.dir_models_tensorflow = os.environ["TFHUB_CACHE_DIR"]         
        self.dir_models_docling = os.environ["DOCLING_MODELS"]
        
        # wordslab-notebooks default models
        self.default_model_chat = os.environ["OLLAMA_CHAT_MODEL"]
        self.default_model_embedding = os.environ["OLLAMA_EMBED_MODEL"]
        self.default_model_code = os.environ["OLLAMA_CODE_MODEL"]
        self.default_model_autocomplete = os.environ["OLLAMA_COMPLETION_MODEL"]



"""Functions to gather context for your LLM from the current notebook cells, files, documents, urls."""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/01_context.ipynb.

# %% auto 0
__all__ = ['get_notebook_data', 'get_mime_text', 'cell2out', 'cell2xml', 'nb2xml', 'get_notebook_context']

# %% ../nbs/01_context.ipynb 2
import asyncio
from ipykernel.comm import Comm

import nbformat
from fastcore.utils import *
from fastcore.xml import to_xml, Src, Source,Out,Outs,Cell

# %% ../nbs/01_context.ipynb 11
def _notebook_data():
    future = asyncio.Future()
    
    def on_msg(msg):
        if not future.done():
            future.set_result(msg['content']['data'])
    
    comm = Comm(target_name='wordslab_notebook_comm', show_warning=False)
    comm.on_msg(on_msg)
    comm.send({'request': 'get_notebook_data'})

    return future

async def get_notebook_data(timeout=1):
    future = _notebook_data()
    try:
        return await asyncio.wait_for(future, timeout=timeout)
    except asyncio.TimeoutError:
        try:
            future = _notebook_data()
            return await asyncio.wait_for(future, timeout=timeout)
        except asyncio.TimeoutError:   
            raise TimeoutError("Failed to receive notebook context from Jupyterlab frontend: install wordslab-notebooks-lib extension, or increase the timeout parameter in seconds, or try to refresh the web page.")

# %% ../nbs/01_context.ipynb 20
def get_mime_text(data):
    "Get text from MIME bundle, preferring markdown over plain"
    if 'text/markdown' in data: return ''.join(list(data['text/markdown']))
    if 'text/plain' in data: return ''.join(list(data['text/plain']))

# %% ../nbs/01_context.ipynb 21
def cell2out(o):
    "Convert single notebook output to XML format"
    if hasattr(o, 'data'): 
        txt = get_mime_text(o.data)
        if txt: return Out(txt, mime='markdown' if 'text/markdown' in o.data else 'plain')
    if hasattr(o, 'text'):
        txt = o.text if isinstance(o.text, str) else ''.join(o.text)
        return Out(txt, type='stream', name=o.get('name', 'stdout'))
    if hasattr(o, 'ename'): return Out(f"{o.ename}: {o.evalue}", type='error')

# %% ../nbs/01_context.ipynb 22
def cell2xml(cell):
    "Convert notebook cell to concise XML format"
    cts = Source(''.join(cell.source)) if hasattr(cell, 'source') and cell.source else None
    out_items = L(getattr(cell,'outputs',[])).map(cell2out).filter()
    outs = []
    if out_items: outs = Outs(*out_items)
    parts = [p for p in [cts, outs] if p]
    return Cell(*parts, type=cell.cell_type)

# %% ../nbs/01_context.ipynb 23
def nb2xml(nb, until_cell_id):
    cells_xml = []
    for c in nb.cells:
        if c.id == until_cell_id: break
        if c.cell_type in ('code','markdown'):
            cells_xml.append(to_xml(cell2xml(c), do_escape=False))
    return '\n'.join(cells_xml)     

# %% ../nbs/01_context.ipynb 25
async def get_notebook_context(timeout=0.5):
    data = await get_notebook_data(timeout=timeout)
    notebook_content = data["notebook"]
    nb = nbformat.from_dict(notebook_content)
    cell_id = data["cell_id"]
    return nb2xml(nb, cell_id)
